/***************************************************************************
                          ipsorcery.c  -  main for console
                             -------------------
    begin                : Sun Sep 23 2001
    copyright            : (C) 2001 by Josiah Zayner
    email                : ajz023@motorola.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include "ingredients.h"



void usage(char *progname)
{

  printf("\nUsage: %s [options]\n"
         "IP: [-is|-id|-ih|-iv|-il|-it|-io|-id|-ip]\n"
         "-is: source host or address def. 127.0.0.1\n"
         "-id: source destination or adress def. 127.0.0.1\n"
         "-ih: IP header length def. 5\n"
         "-iv: IP version def. 4\n"
         "-il: Time-to-Live def. 64\n"
         "-it: Type-of-Service def. 0\n"
         "-io: IP frag offset [(D)on't Fragment|(M)ore Fragments|(F)ragment|(N)one]\n"
         "-ii: IP packet ID for fragmentaion def. 0\n"
         "-ip: IP protocol [TCP|UDP|ICMP|IP] def. TCP\n"
         "TCP: [-ts|-td|-to|-tq|-ta|-tf|-tw|-tu]\n"
         "-ts: TCP source port, def. rand()\n"
         "-td: TCP destination port def. 80\n"
         "-to: TCP data offset of header def. 5\n"
         "-tq: TCP sequence number def. rand()\n"
         "-ta: TCP ack sequence number def. 0\n"
         "-tf: TCP flags [(S)yn|(A)ck|(F)in|(P)ush|(R)st|(U)rg|(N)one] def. S\n"
         "-tw: TCP Window Size def. rand()\n"
         "-tu: TCP urg pointer def. 0\n"
         "UDP: [-us|-ud|-ul]\n"
         "-us: UDP source port def. rand()\n"
         "-ud: UDP destination port def. 161\n"
         "-ul: UDP length\n"
         "ICMP: [-ct|-cs]\n"
         "-ct: ICMP type def. ECHO REQUEST\n"
         "-cs: ICMP sub code def. 0\n"
         "For list of ICMP Types and Subcodes run prpogram with -h icmp.\n"
         "-D \"<data>\": for datapayload\n"
         ,progname
        );

  exit(-1);
}

void icmp_info(void)
{
  printf("ICMP Types and Subcodes:\n"
         "Types:\n"
         "Echo Reply                   0\n"
         "Destination Unreachable      3\n"
         "  Subcodes:\n"
         "     Network Unreachable   0\n"
         "     Host Unreachable      1\n"
         "     Protocol Unreachable  2\n"
         "     Port Unreachable      3\n"
         "     Fragmentation Needed  4\n"
         "     Source Route Failed   5\n"
         "     Network Unknown       6\n"
         "     Host Unknown          7\n"
         "     Host Isolated         8\n"
         "     Network Prohibited    9\n"
         "     Host Prohibited      10\n"
         "     Bad TOS for Network  11\n"
         "     Bad TOS for Host     12\n"
         "     Packet Filtered      13\n"
         "     Precedence Violation 14\n"
         "     Precedence Cutoff    15\n"
         "Source Quench                4\n"
         "Redirect                     5\n"
         "  Subcodes:\n"
         "     Redirect Network      0\n"
         "     Redirect Host         1\n"
         "     Redirect Network TOS  2\n"
         "     Redirect Host TOS     3\n"
         "Echo Request                 8\n"
         "Time Exceeded               11\n"
         "  Subcodes:\n"
         "     Time to Live          0\n"
         "     Frag Reassembly       1\n"
         "Parameter Problems          12\n"
         "  Subcode:\n"
         "     Option Missing        0\n"
         "Timestamp Request           13\n"
         "Timestamp Reply             14\n"
         "Information Request         15\n"
         "Information Reply           16\n"
         "Address Mask Request        17\n"
         "Address Mask Reply          18\n"
        );
  exit(0);

}

int main(int argc, char **argv)
{

  int x = 1, tcp_go = 0, udp_go = 0, icmp_go = 0;
  int d_size = 0;
  struct protoent *ourproto;
  struct ip *eyep = clalloc(sizeof(struct ip));
  char *pack = NULL;
  struct tcphdr *tcpea = NULL;
  struct udphdr *udpea = NULL;
  struct icmp *icmpea = NULL;
  char *data = NULL;

  if(argc <= 1){ usage(argv[0]); }  /* if no arguments print usage */

  srand(time(NULL)); /* seed current time for random numbers*/

  /* Everyone of our packets use IP! */
  ip_sorcery(eyep, wherefromto("127.0.0.1"), wherefromto("127.0.0.1"), 6, 5, 4, 0, 40, 0, 64, 0);

/* if we got one argument lets parse it*/
if(argc > 1)
{

for(;x < argc; x++)
{

  if(argv[x][0] != '-')
  {
    fprintf(stderr, "Bad Option: %s\n",argv[x]);
    usage(argv[0]);
  }

  switch((int)argv[x][1])
  {

    case 'i':

     switch((int)argv[x][2])
     {
       case 's':
       eyep->ip_src = wherefromto(argv[x+1]);
       break;

       case 'd':
       eyep->ip_dst = wherefromto(argv[x+1]);
       break;

       case 'h':
       eyep->ip_hl = atoi(argv[x+1]);
       break;

       case 'v':
       eyep->ip_v = atoi(argv[x+1]);
       break;

       case 'l':
       eyep->ip_ttl = atoi(argv[x+1]);
       break;

       case 't':
       eyep->ip_tos = atoi(argv[x+1]);
       break;

       case 'o':
       if(strstr(argv[x + 1], "F"))
       {  eyep->ip_off = 16; }
       else if(strstr(argv[x + 1], "D"))
       { eyep->ip_off = 64; }
       else if(strstr(argv[x + 1], "M"))
       { eyep->ip_off = 32; }
       else if(strstr(argv[x + 1], "N"))
       { eyep->ip_off = 0; }
       break;

       case 'i':
       eyep->ip_id = atoi(argv[x+1]);
       break;

       case 'p':
         if((ourproto = getprotobyname(argv[x+1])) == NULL)
         {
           perror("Getprotobyname:");
           exit(-1);
         }
         eyep->ip_p = ourproto->p_proto;
       break;
    }
    x++; /* increment x so we skip the next argument */
    break;

    case 't':
    if(icmp_go || udp_go){ fprintf(stderr, "Only one protocol at a time please!\n"); exit(-1); }

    /* intialize tcp packet */
    else if(!tcp_go)
    {
      d_size = 65535 - (sizeof(struct ip) + sizeof(struct tcphdr));
      pack = clalloc(sizeof(struct ip) + sizeof(struct tcphdr) + d_size);
      tcpea = clalloc(sizeof(struct tcphdr));
      tcpea->th_sport = htons(rand() % 3000);  /* random source port <= 3000 */
      tcpea->th_dport = htons(23);
      tcpea->th_off = 5;
      tcpea->th_seq = htonl(rand() % 9000000000); /* sequence number */
      tcpea->th_ack = 0;    /* ack sequence */
      tcpea->th_flags = TH_SYN; /* default tcp SYN packet */
      tcpea->th_win = htons((rand() % 1024) + 300); /* 300 <= random window size <= 1324 */
      tcpea->th_urp = 0;
      tcp_go++; /* so we don't intialize again */
    }


      switch((int)argv[x][2])
      {
        case 's':
        tcpea->th_sport = htons(atoi(argv[x+1]));
        break;

        case 'd':
        tcpea->th_dport = htons(atoi(argv[x+1]));
        break;

        case 'o':
        tcpea->th_off = atoi(argv[x+1]);
        break;

        case 'q':
        tcpea->th_seq = htonl(atoi(argv[x+1]));
        break;

        case 'a':
        tcpea->th_ack = htonl(atoi(argv[x+1]));
        break;

        case 'f':
        tcpea->th_flags = 0;
        if(strchr(argv[x+1], 'S'))
        { tcpea->th_flags += TH_SYN; }
        if(strchr(argv[x+1], 'A'))
        { tcpea->th_flags += TH_ACK; }
        if(strchr(argv[x+1], 'F'))
        { tcpea->th_flags += TH_FIN; }
        if(strchr(argv[x+1], 'R'))
        { tcpea->th_flags += TH_RST; }
        if(strchr(argv[x+1], 'U'))
        { tcpea->th_flags += TH_URG; }
        if(strchr(argv[x+1], 'P'))
        { tcpea->th_flags += TH_PUSH; }
        if(strchr(argv[x+1], 'N'))
        { tcpea->th_flags = 0; }
        break;

        case 'w':
        tcpea->th_win = htons(atoi(argv[x+1]));
        break;

        case 'u':
        tcpea->th_urp = atoi(argv[x+1]);
        break;

      }


    x++;
    break;

    case 'u':
    if(tcp_go || icmp_go){ fprintf(stderr, "Only one protocol at a time please!\n"); exit(-1); }

    /* intialize packet a max data size */
    else if(!udp_go)
    {
      d_size = 65535 - (sizeof(struct ip) + sizeof(struct udphdr));  /* max datat size */
      pack = clalloc(sizeof(struct ip) + sizeof(struct udphdr) + d_size);  /* allocate memory for packet */
      udpea = clalloc(sizeof(struct udphdr));
      udpea->uh_sport = htons(rand() % 3000);  /* source port */
      udpea->uh_dport = htons(20);   /* destination port */
      udpea->uh_ulen = sizeof(struct udphdr);
      udp_go++;     /* so we don't intialize again */
    }

      switch((int)argv[x][2])
      {
        case 's':
        udpea->uh_sport = htons(atoi(argv[x + 1]));
        break;

        case 'd':
        udpea->uh_dport = htons(atoi(argv[x + 1]));

        case 'u':
        udpea->uh_ulen = sizeof(struct udphdr);
        break;

      }
    x++;
    break;

    case 'c':

      if(tcp_go || udp_go){ fprintf(stderr, "Only one protocol at a time please!\n"); exit(-1); }

      /* intialize our ICMP header */
      else if(!icmp_go)
      {
        d_size = 65535 - (sizeof(struct ip) + sizeof(struct icmp));
        pack = clalloc(sizeof(struct ip) + sizeof(struct icmp) + d_size);
        icmpea = clalloc(sizeof(struct icmp));
        icmpea->icmp_type = ICMP_ECHO;
        icmpea->icmp_code = 0;
        icmp_go++;
      }
      switch((int)argv[x][2])
      {
        /* should we error check our type and subcodes?
           this prevents experimenting our codes and types I missed..
        */
        case 't':
        icmpea->icmp_type = atoi(argv[x + 1]);
        break;

        case 's':
        icmpea->icmp_code = atoi(argv[x + 1]);
        break;

      }
    x++;
    break;

    /* get data and make sure it is small enough for one packet */
    case 'D':
    if(argv[x + 1])
    {
      data = clalloc(d_size);
      if(strlen(argv[x + 1]) > d_size){ prterr("Data size too big!\n"); exit(-1); }
      memcpy(data, argv[x + 1], strlen(argv[x + 1]));
      d_size = strlen(argv[x + 1]);
    }
    x++;
    break;

    case 'h':
    if(!argv[x + 1]){ usage(argv[0]); }
    else if(strstr(argv[x + 1],"icmp")){ icmp_info(); }
    break;

    default:
      fprintf(stderr, "Bad Option: %s\n", argv[x]);
      usage(argv[0]);
    break;

  }/* switch argv[x][1] */

} /* for(... */

}/* if(argc > 1) */

if(!data){ d_size = 0; }
/* hit up our main functions */
  if(tcp_go)
  {
    tcp_sorcery(tcpea, eyep, pack, data, d_size);
    free(tcpea);
  }

  else if(udp_go)
  {
    eyep->ip_p = 17;
    udp_sorcery(udpea, eyep, pack, data, d_size);
    free(udpea);
  }

  else if(icmp_go)
  {
    eyep->ip_p = 1;
    icmp_sorcery(icmpea, eyep, pack, data, d_size);
    free(icmpea);
  }

  if(data){ free(data); }
  free(eyep);
  free(pack);

  return 0;
}


