/***************************************************************************
                          sorcery.c  -  for sorcery and GUI--
                             -------------------
    begin                : Tue Sep 11 2001
    copyright            : (C) 2001 by Josiah Zayner
    email                : ajz023@motorola.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "ingredients.h"



void ip_sorcery(struct ip *ip_sorc,
                struct in_addr source,
                struct in_addr dest,
                int proto,
                unsigned int hl,
                unsigned int ver,
                u_char tos,
                u_short len,
                u_short id,
                u_char ttl,
                int off
               )
{
  ip_sorc->ip_hl = hl;
  ip_sorc->ip_v = ver;
  ip_sorc->ip_tos = tos;
  ip_sorc->ip_len = len;
  ip_sorc->ip_id = id;
  ip_sorc->ip_ttl = ttl;
  ip_sorc->ip_p = proto;
  ip_sorc->ip_off = off;
  ip_sorc->ip_src = source;
  ip_sorc->ip_dst = dest;

}

int tcp_sorcery(struct tcphdr *tcp_magic, struct ip *ip_magic, char *pack, char *datap, int dps)
{
  char *envelope = clalloc(sizeof(struct ip) + sizeof(struct tcphdr) + dps);
  struct nmrhdr *nmr_magic = clalloc(sizeof(struct nmrhdr));
  /*
  int s, bytes;
  char buff[1024];
  struct sockaddr_in endup;
  struct ip *iprh = clalloc(sizeof(struct ip));
  */

  /* putting tcp header and data in packet for checksum */
  memcpy(envelope + sizeof(struct nmrhdr), tcp_magic, sizeof(struct tcphdr));
  memcpy(envelope + sizeof(struct nmrhdr) + sizeof(struct tcphdr), datap, dps);

  /* create psuedo header */
  nmr_magic->saddr = ip_magic->ip_src;
  nmr_magic->daddr = ip_magic->ip_dst;
  nmr_magic->none = 0;
  nmr_magic->protocol = 6;
  nmr_magic->length = htons(sizeof(struct tcphdr) + dps);
  memcpy(envelope, nmr_magic, sizeof(struct nmrhdr));

  /* calculate checksum for the psuedo no miss routed packet header and tcp header */
  tcp_magic->th_sum = in_cksum((u_short *)envelope, sizeof(struct tcphdr) + sizeof(struct nmrhdr) + dps);
  free(envelope);
  free(nmr_magic);

  /* putting our tcp header in the final packet */
  memcpy(pack + sizeof(struct ip) + sizeof(struct tcphdr), datap, dps);
  memcpy(pack + sizeof(struct ip), tcp_magic, sizeof(struct tcphdr));
  memcpy(pack, ip_magic, sizeof(struct ip));

  /* send packet */
  if(cast_spell(pack,(sizeof(struct ip) + sizeof(struct tcphdr) + dps), tcp_magic->th_dport, ip_magic->ip_dst) == -1)
  {
     perror("sendto");
     return -1;
  }
  /*
  if(socket(s, SOCK_PACKET, 0x0003) == -1)
  {
    perror("socket");
    exit -1;
  }

  endup.sin_family = PF_INET;
  endup.sin_port = tcp_magic->th_dport;
  endup.sin_addr = ip_magic->ip_dst;

  if((s = socket(PF_INET, SOCK_RAW, ETH_P_ALL)) == -1){ perror("socket1"); exit(-1); }
  bytes = recvfrom(s, buff, 1024, 0, (struct sockaddr *)&endup, (int *)sizeof(struct sockaddr));
  if(bytes <= 0)
  {
     perror("recvfrom");
     exit(-1);
  }
  memcpy(iprh, buff, sizeof(struct ip));
  printf("IP Version:%d\n\n ",iprh->ip_v);
  free(iprh);
  */
  return 0;
}


int udp_sorcery(struct udphdr *udpp, struct ip *ip_magic, char *pack, char *datap, int dps)
{
  char *envelope = clalloc(sizeof(struct nmrhdr) + sizeof(struct udphdr));
  struct nmrhdr *nmr_magic = clalloc(sizeof(struct nmrhdr));

  memcpy(envelope + sizeof(struct nmrhdr), udpp, sizeof(struct udphdr));
  memcpy(envelope + sizeof(struct nmrhdr) + sizeof(struct udphdr), datap, dps);

  nmr_magic->saddr = ip_magic->ip_src;
  nmr_magic->daddr = ip_magic->ip_dst;
  nmr_magic->none = 0;
  nmr_magic->protocol = 6;
  nmr_magic->length = htons(sizeof(struct udphdr) + dps);
  memcpy(envelope, nmr_magic, sizeof(struct nmrhdr));

  /* calculate checksum for the psuedo no miss routed packet header and tcp header */
  udpp->uh_sum = in_cksum((u_short *)envelope, sizeof(struct udphdr) + sizeof(struct nmrhdr) + dps);
  free(envelope);
  free(nmr_magic);

  /* putting our udp header in the packet */
  memcpy(pack + sizeof(struct ip) + sizeof(struct udphdr), datap, dps);
  memcpy(pack + sizeof(struct ip), udpp, sizeof(struct udphdr));
  memcpy(pack, ip_magic, sizeof(struct ip));


  if(cast_spell(pack,(sizeof(struct ip) + sizeof(struct udphdr) + dps), udpp->uh_dport, ip_magic->ip_dst) == -1)
  {
     perror("sendto");
     return -1;
  }


  return 0;

}

int icmp_sorcery(struct icmp *icmp_magic, struct ip *ip_magic, char *pack, char *datap, int dps)
{

  memcpy(pack, icmp_magic, sizeof(struct icmp));
  memcpy(pack + sizeof(struct icmp), datap, dps);
  icmp_magic->icmp_cksum = in_cksum((u_short *)pack, sizeof(struct icmp) + dps);

  memcpy(pack + sizeof(struct ip) + sizeof(struct icmp), datap, dps);
  memcpy(pack, ip_magic, sizeof(struct ip));
  memcpy(pack + sizeof(struct ip), icmp_magic, sizeof(struct icmp));
  if(cast_spell(pack,(sizeof(struct ip) + sizeof(struct icmp) + dps), 0, ip_magic->ip_dst) == -1)
  {
     perror("sendto");
     return -1;
  }
  return 0;
}


int cast_spell(char *envelope, int env_size, u_int8_t deport, struct in_addr src)
{
  int s, yes = 1;
  struct sockaddr_in endup;

 /* open a raw socket */
  if((s = socket(PF_INET, SOCK_RAW, IPPROTO_RAW)) == -1)
  {
    perror("socket");
    exit(s);
  }

  /* set socket options at IP level, so we can create our own IP header */
  if((setsockopt(s, SOL_IP, IP_HDRINCL, &yes, sizeof(yes))) == -1)
  {
    perror("setsockopt");
    exit(-1);
  }

  /* fill in sockaddr struct bLAH! */
  memset(&endup, '\0', sizeof(struct sockaddr_in));
  endup.sin_family = PF_INET;
  endup.sin_port = deport;
  endup.sin_addr = src;

  /* send packet woohooo! */
  sendto(s, envelope, env_size, 0, (struct sockaddr *)&endup, sizeof(struct sockaddr));

  return s;
}


void prterr(char *message)
{ fprintf(stderr, message); }

