/***************************************************************************
                          spellbook.c  -  Functions for our spells
                             -------------------
    begin                : Wed Jul 25 2001
    copyright            : (C) 2001 by Josiah Zayner
    email                : ajz023@motorola.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "magic.h"



GtkWidget *make_combo(GtkWidget *box,
                      int obj_len,
                      int obj_hi,
                      short int pos_len,
                      short int pos_hi
                     )
{
  /* our combo box */
  box = gtk_combo_new();
  /* show the widget */
  gtk_widget_show(box);
  /* place widget in fixed position */
  gtk_fixed_put(GTK_FIXED(fixed_pos), box, pos_len, pos_hi);
  /* set size of widget */
  gtk_widget_set_usize(box, obj_len, obj_hi);
  /* item must be in list, and item cannot be blank */
  gtk_combo_set_value_in_list(GTK_COMBO(box), TRUE, FALSE);

  return box;
}

GtkWidget *make_label(GtkWidget *lbl,
                      char *text,
                      short int pos_len,
                      short int pos_hi
                     )
{
  /* create a label with 'text' on it */
  lbl = gtk_label_new(text);
  gtk_widget_show(lbl);
  /* position label */
  gtk_fixed_put(GTK_FIXED(fixed_pos), lbl, pos_len, pos_hi);
  return lbl;
}

GtkWidget *make_button(GtkWidget *button,
                       char *text,
                       int obj_len,
                       int obj_hi,
                       short int pos_len,
                       short int pos_hi,
                       GtkWidget *fixed1
                      )
{
  button = gtk_button_new_with_label(text);
  gtk_widget_show(button);
  /* place widget in fixed position */
  gtk_fixed_put(GTK_FIXED(fixed1), button, pos_len, pos_hi);
  /* set size of widget */
  gtk_widget_set_usize(button, obj_len, obj_hi);
  return button;
}

GtkWidget *make_text(GtkWidget *text,
                     int obj_len,
                     int obj_hi,
                     short int pos_len,
                     short int pos_hi
                    )
{


  text = gtk_entry_new();
  gtk_widget_set_usize(text, obj_len, obj_hi);
  gtk_fixed_put(GTK_FIXED(fixed_pos), text, pos_len, pos_hi);
  gtk_entry_set_editable(GTK_ENTRY(text), TRUE);
  gtk_widget_show(text);

  return text;

}


GtkWidget *make_check(GtkWidget *check,
                      const char *label,
                      short int pos_len,
                      short int pos_hi
                     )
{

  check = gtk_check_button_new_with_label(label);
  gtk_fixed_put(GTK_FIXED(fixed_pos), check, pos_len, pos_hi);
  gtk_widget_show(check);
  return check;
}


int geticmptypebyname(const char *type_name)
{
  int type = 0;

  if(strstr(type_name, "Echo Reply"))
  { type = ICMP_ECHOREPLY; }

  else if(strstr(type_name, "Destination Unreachable"))
  { type = ICMP_DEST_UNREACH; }

  else if(strstr(type_name, "Source Quench"))
  { type = ICMP_SOURCE_QUENCH; }

  else if(strstr(type_name, "Redirect"))
  { type = ICMP_REDIRECT; }

  else if(strstr(type_name, "Echo Request") && !strstr(type_name, "Echo Reply"))
  { type = ICMP_ECHO; }
  else if(strstr(type_name, "Router Advertisement"))
  { type = ICMP_ROUTERADVERT; }

  else if(strstr(type_name, "Router Solicitation"))
  { type = ICMP_ROUTERSOLICIT; }

  else if(strstr(type_name, "Time Exceeded"))
  { type = ICMP_TIMXCEED; }

  else if(strstr(type_name, "Parameter Problem"))
  { type = ICMP_PARAMPROB; }

  else if(strstr(type_name, "Timestamp Request"))
  { type = ICMP_TSTAMP; }

  else if(strstr(type_name, "Timestamp Reply"))
  { type = ICMP_TSTAMPREPLY; }

  else if(strstr(type_name, "Information Request"))
  { type = ICMP_IREQ; }

  else if(strstr(type_name, "Information Reply"))
  { type = ICMP_IREQREPLY; }

  else if(strstr(type_name, "Address Mask Request"))
  { type = ICMP_MASKREQ; }

  else if(strstr(type_name, "Address Mask Reply"))
  { type = ICMP_MASKREPLY; }



  return type;
}



int geticmpsubbyname(const char *sub_name)
{
  int sub = 0;

  if(strstr(sub_name, "Network Unreachable") || strstr(sub_name, "Network Redirect")
     || strstr(sub_name, "In Transit"))
  { sub = 0; }

  else if(strstr(sub_name, "Host Unreachable") || strstr(sub_name, "Host Redirect")
          || strstr(sub_name, "In Reass") || strstr(sub_name, "Required Option Absent"))
  { sub = 1; }

  else if(strstr(sub_name, "Protocol Unreachable") || strstr(sub_name, "Network TOS Redirect"))
  { sub = 2; }

  else if(strstr(sub_name, "Port Unreachable") || strstr(sub_name, "Host TOS Redirect"))
  { sub = 3; }

  else if(strstr(sub_name, "Need Fragmentation"))
  { sub = ICMP_UNREACH_NEEDFRAG; }

  else if(strstr(sub_name, "Source Route Failed"))
  { sub = ICMP_UNREACH_SRCFAIL; }

  else if(strstr(sub_name, "Network Unknown"))
  { sub = ICMP_UNREACH_NET_UNKNOWN; }

  else if(strstr(sub_name, "Host Unknown"))
  { sub = ICMP_UNREACH_HOST_UNKNOWN; }

  else if(strstr(sub_name, "Host Isolated"))
  { sub = ICMP_UNREACH_ISOLATED; }

  else if(strstr(sub_name, "Network Prohibited"))
  { sub = ICMP_UNREACH_NET_PROHIB; }

  else if(strstr(sub_name, "Host Prohibited"))
  { sub = ICMP_UNREACH_HOST_PROHIB; }

  else if(strstr(sub_name, "Bad TOS For Network"))
  { sub = ICMP_UNREACH_TOSNET; }

  else if(strstr(sub_name, "Bad TOS For Host"))
  { sub = ICMP_UNREACH_TOSHOST; }

  else if(strstr(sub_name, "Packet Filtered"))
  { sub = ICMP_UNREACH_FILTER_PROHIB; }

  else if(strstr(sub_name, "Host Precedence Violation"))
  { sub = ICMP_UNREACH_HOST_PRECEDENCE; }

  else if(strstr(sub_name, "Precedence Cutoff"))
  { sub = ICMP_UNREACH_PRECEDENCE_CUTOFF; }

  return sub;
}


GtkWidget *make_te(GtkWidget *text,
                   int obj_len,
                   int obj_hi,
                   short int pos_len,
                   short int pos_hi
                  )
{


  text = gtk_text_new(NULL, NULL);
  gtk_widget_set_usize(text, obj_len, obj_hi);
  gtk_fixed_put(GTK_FIXED(fixed_pos), text, pos_len, pos_hi);
  gtk_text_set_editable(GTK_TEXT(text), TRUE);
 /*  gtk_text_set_line_wrap(GTK_TEXT(text), TRUE);   */
  gtk_widget_show(text);

  return text;

}









































